@@ -1,26 +1,35 @@
 import json
 import os
+import pwd
 import random
 import string
 import time
 from datetime import datetime
+from enum import Enum
 from pathlib import Path
 from typing import Dict, Optional
 
+try:
+    from functools import cached_property
+except ImportError:
+    # Fallback for Python 3.7
+    from backports.cached_property import cached_property
+
 import humanize
 import psutil
 
 from hapless import config
-from hapless.utils import allow_missing, logger
+from hapless.utils import allow_missing, get_mtime, logger
 
-"""
-active
-* paused
-* running
-finished
-* finished(failed) non-zero rc
-* finished(success) zero rc
-"""
+
+# TODO: add unbound status for the hap without associated pid
+class Status(str, Enum):
+    # Active statuses
+    PAUSED = "paused"
+    RUNNING = "running"
+    # Finished statuses
+    FAILED = "failed"
+    SUCCESS = "success"
 
 
 class Hap(object):
@@ -31,8 +40,11 @@ class Hap(object):
         name: Optional[str] = None,
         cmd: Optional[str] = None,
     ):
+        if not hap_path.is_dir():
+            raise ValueError(f"Path {hap_path} is not a directory")
+
         self._hap_path = hap_path
-        self._hid = os.path.basename(hap_path)
+        self._hid: str = hap_path.name
 
         self._pid_file = hap_path / "pid"
         self._rc_file = hap_path / "rc"
@@ -40,24 +52,27 @@ class Hap(object):
         self._cmd_file = hap_path / "cmd"
         self._env_file = hap_path / "env"
 
-        self._set_name(name)
+        self._set_raw_name(name)
         self._set_cmd(cmd)
 
-    def _set_name(self, name: Optional[str]):
-        """
-        Sets name for the first time on hap creation.
-        """
-        if name is None:
-            suffix = self.get_random_name()
-            name = f"hap-{suffix}"
+    def set_name(self, name: str):
+        with open(self._name_file, "w") as f:
+            f.write(name)
 
-        if self.name is None:
-            with open(self._name_file, "w") as f:
-                f.write(name)
+    def _set_raw_name(self, raw_name: Optional[str]):
+        """
+        Set name for the first time on hap creation.
+        """
+        if raw_name is None:
+            suffix = self.get_random_name()
+            raw_name = f"hap-{suffix}"
+
+        if self.raw_name is None:
+            self.set_name(raw_name)
 
     def _set_cmd(self, cmd: Optional[str]):
         """
-        Sets cmd for the first tiem on hap creation.
+        Set cmd for the first time on hap creation.
         """
         if self.cmd is None:
             if cmd is None:
@@ -89,7 +104,7 @@ class Hap(object):
         with open(self._env_file, "w") as env_file:
             env_file.write(json.dumps(environ))
 
-    def attach(self, pid: int):
+    def bind(self, pid: int):
         """
         Associate hap object with existing process by pid
         """
@@ -97,10 +112,10 @@ class Hap(object):
             self._set_pid(pid)
             self._set_env()
         except (RuntimeError, psutil.AccessDenied) as e:
-            logger.error(f"Cannot attached due to {e}")
+            logger.error(f"Cannot bind due to {e}")
 
     @staticmethod
-    def get_random_name(length: int = 6):
+    def get_random_name(length: int = 6) -> str:
         return "".join(
             random.sample(
                 string.ascii_lowercase + string.digits,
@@ -109,22 +124,25 @@ class Hap(object):
         )
 
     # TODO: add extended status to show panel proc.status()
-    # TODO: return enum entries instead
     @property
-    def status(self) -> str:
+    def status(self) -> Status:
         proc = self.proc
         if proc is not None:
             if proc.status() == psutil.STATUS_STOPPED:
-                return "paused"
-            return "running"
+                return Status.PAUSED
+            return Status.RUNNING
 
         if self.rc != 0:
-            return "failed"
+            return Status.FAILED
 
-        return "success"
+        return Status.SUCCESS
 
-    @property
+    @cached_property
     def proc(self):
+        # NOTE: this is cached for the instance lifetime, fits our use case
+        if self.pid is None:
+            return
+
         try:
             return psutil.Process(self.pid)
         except psutil.NoSuchProcess as e:
@@ -145,11 +163,12 @@ class Hap(object):
     @property
     def runtime(self) -> str:
         proc = self.proc
+        runtime = 0
         if proc is not None:
             runtime = time.time() - proc.create_time()
-        else:
-            start_time = os.path.getmtime(self._pid_file)
-            finish_time = os.path.getmtime(self._rc_file)
+        elif self._pid_file.exists():
+            start_time = get_mtime(self._pid_file)
+            finish_time = get_mtime(self._rc_file) or get_mtime(self.stderr_path)
             runtime = finish_time - start_time
 
         return humanize.naturaldelta(runtime)
@@ -175,7 +194,7 @@ class Hap(object):
         return self.proc is not None
 
     @property
-    def hid(self) -> int:
+    def hid(self) -> str:
         return self._hid
 
     @property
@@ -196,23 +215,55 @@ class Hap(object):
 
     @property
     @allow_missing
-    def name(self) -> Optional[str]:
+    def raw_name(self) -> Optional[str]:
         with open(self._name_file) as f:
             return f.read().strip()
 
+    @cached_property
+    def name(self) -> str:
+        """
+        Base name without restarts counter.
+        """
+        return self.raw_name.split(config.RESTART_DELIM)[0]
+
+    @cached_property
+    def restarts(self) -> int:
+        _, *rest = self.raw_name.rsplit("@", maxsplit=1)
+        return int(rest[0]) if rest else 0
+
     @property
-    def path(self):
+    def path(self) -> Path:
         return self._hap_path
 
     @property
-    def stdout_path(self):
+    def stdout_path(self) -> Path:
         return self._hap_path / "stdout.log"
 
     @property
-    def stderr_path(self):
+    def stderr_path(self) -> Path:
         return self._hap_path / "stderr.log"
 
-    def __str__(self):
+    @property
+    def accessible(self) -> bool:
+        """
+        Check if current user has control over the hap.
+        """
+        try:
+            os.utime(self.path)
+            return True
+        except PermissionError:
+            return False
+
+    @property
+    def owner(self) -> str:
+        stat = self.path.stat()
+        try:
+            owner = pwd.getpwuid(stat.st_uid).pw_name
+        except KeyError:
+            owner = f"{stat.st_uid}:{stat.st_gid}"
+        return owner
+
+    def __str__(self) -> str:
         return f"#{self.hid} ({self.name})"
 
     def __rich__(self) -> str:
